@@ -80,10 +80,11 @@ class HypergraphLP:
     def path(self):
         if self._status != pulp.LpStatusOptimal:
             raise Exception("No optimal solution.")
-        path_edges = [edge
-                      for edge in self.hypergraph.edges
-                      if pulp.value(self.edge_vars[edge.id]) == 1.0]
-        return ph.Path(self.hypergraph, path_edges)
+        else:
+            path_edges = [edge
+                          for edge in self.hypergraph.edges
+                          if pulp.value(self.edge_vars[edge.id]) == 1.0]
+            return ph.Path(self.hypergraph, path_edges)
 
     @property
     def objective(self):
@@ -95,6 +96,12 @@ class HypergraphLP:
                 for edge in self.hypergraph.edges
                 if pulp.value(self.edge_vars[edge.id]) > 0.0}
 
+    def decode_fractional(self):
+        vec = [pulp.value(self.edge_vars[edge.id])
+               for edge in self.hypergraph.edges]
+        weights = ph.LogViterbiPotentials(self.hypergraph).from_vector(vec)
+        return ph.best_path(self.hypergraph, weights)
+
     def add_constraints(self, constraints):
         """
         Add hard constraints to the hypergraph.
