@@ -61,8 +61,8 @@ def _subgradient(graph, weight_potentials, potentials):
         score = dual_weights.dot(path)
         vec = potentials.dot(path)
         subgrad = np.zeros(len(x))
-        for i in vec:
-            subgrad[i] = vec[i]
+        for i, j in vec:
+            subgrad[i] = j
         return score, subgrad, path
     return fn
 
